#!/usr/bin/env python3

import argparse
from multiprocessing import Pool, Queue
from multiprocessing import Manager
from pathlib import Path
import os


def serch_big_files_worker(task_num):
    # get max size and convert it to bytes
    max_size = int(envi.get('max_size'))*1024*1024
    while True:
        # get path from queue
        working_path = unsearched_dirs.get()
        # ###  find big files block
        # get all filenames from current dir
        files = [file.name for file in os.scandir(working_path) if file.is_file(follow_symlinks=False)]
        # for each file
        for file in files:
            # make full path for each file
            file_path = os.path.join(working_path, file)
            # check size and if more then max size, then appent to oversized files list.
            if os.path.getsize(file_path) > max_size:
                oversized_files.append(str(file_path))

        # ### add subdirs to queue block
        # find dir names in current directory
        sub_dirs = [dir.name for dir in os.scandir(working_path) if dir.is_dir(follow_symlinks=False)]
        # for each dir
        for dirname in sub_dirs:
            # make full path of dir
            dir_path = os.path.join(working_path, dirname)
            # put dir to queue
            unsearched_dirs.put(dir_path)
        # when we all maked, close thread
        unsearched_dirs.task_done()


def search_big_files_start(threads):
    # get path from envi dict
    path = envi.get('path')
    # create a pool for parallel worners
    pool = Pool(threads)
    # add to queue our base path
    unsearched_dirs.put(path)
    # start parallel workes: 1st parameter is worker function, second - is how many thread will be used
    pool.map_async(serch_big_files_worker, range(threads))
    # when all was be done, close pool and make join for queue(its mean end of queue)
    pool.close()
    unsearched_dirs.join()


def main():
    parser = argparse.ArgumentParser(description="""This is a programm for parallel searching files more than 
        maximum size. Programm uses threads for fast finding dirs and subdirs in path(EXCEPT symlinks)""")
    parser.add_argument('--path', help='path, where we will find subdirs and files for check',
                        required=True, type=lambda p: Path(p).absolute())
    parser.add_argument('--max_size', help='maximum file size for finding, in Mb', required=True, type=int)
    parser.add_argument('--threads', help='number of threads for use, default 2', default=2, type=int)
    p = parser.parse_args()
    # lets add to envi dict  some  envs for store and for give them to our parralell threads
    envi.update([
        ('path', str(p.path)),
        ('max_size', str(p.max_size))
    ])
    # use start function for start parallel workers
    search_big_files_start(p.threads)
    # print result
    print("list of file more than %d Mb", p.max_size)
    print(*oversized_files)


if __name__ == "__main__":
    # create envi dict for store some parameter for parallel threads
    envi = Manager().dict()
    # queue for path for check.
    unsearched_dirs = Manager().Queue()
    # list for files whom has size more then max_size
    oversized_files = Manager().list()
    main()